﻿using System;
using System.Linq.Expressions;
using Microsoft.Practices.Prism.Commands;

namespace XCE.SelectionReset
{
    public sealed class DependencyCommand : DelegateCommand, IDisposable
    {
        private DependencyListener _listener;

        public DependencyCommand(Action executeMethod, Expression<Func<bool>> canExecuteMethod)
            : this(executeMethod, Compile(canExecuteMethod), canExecuteMethod)
        {
        }

        private DependencyCommand(Action executeMethod, Func<bool> canExecuteMethod, Expression<Func<bool>> dependencies)
            : base(WrapExecute(executeMethod, canExecuteMethod), canExecuteMethod)
        {
            _listener = new DependencyListener(RaiseCanExecuteChanged);
            _listener.Add(dependencies);
        }

        public void Dispose()
        {
            if (_listener != null)
            {
                _listener.Dispose();
                _listener = null;
            }
        }

        private static Func<bool> Compile(Expression<Func<bool>> canExecuteMethod)
        {
            if (canExecuteMethod == null)
            {
                throw new ArgumentNullException("canExecuteMethod");
            }

            return canExecuteMethod.Compile();
        }

        private static Action WrapExecute(Action executeMethod, Func<bool> canExecuteMethod)
        {
            if (executeMethod == null)
            {
                throw new ArgumentNullException("executeMethod");
            }

            return () =>
            {
                if (canExecuteMethod())
                {
                    executeMethod();
                }
            };
        }
    }
}