﻿using System;
using System.ComponentModel;

namespace XCE.SelectionReset
{
    public partial class NamedItemsHolder<TItem>
    {
        private sealed class ItemPropertyDescriptor : PropertyDescriptor
        {
            public override Type ComponentType { get; }

            public override bool IsReadOnly => true;

            public override Type PropertyType => typeof(TItem);

            public ItemPropertyDescriptor(string name, Type componentType)
                : base(name, null)
            {
                ComponentType = componentType;
            }

            public override bool CanResetValue(object component)
            {
                return false;
            }

            public override object GetValue(object component)
            {
                component.CheckNotNull(nameof(component));

                var holder = component as NamedItemsHolder<TItem>;
                if (holder == null)
                {
                    throw new ArgumentOutOfRangeException(nameof(component));
                }

                return holder[Name];
            }

            public override void ResetValue(object component)
            {
                throw new NotSupportedException();
            }

            public override void SetValue(object component, object value)
            {
                throw new NotSupportedException();
            }

            public override bool ShouldSerializeValue(object component)
            {
                return false;
            }
        }
    }
}
