﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;

namespace XCE.SelectionReset
{
    public partial class NamedItemsHolder<TItem> : CustomTypeDescriptor, INotifyPropertyChanged, IDisposable
    {
        private readonly IDictionary<string, TItem> _items = new Dictionary<string, TItem>(StringComparer.OrdinalIgnoreCase);
        private PropertyChangedEventHandler _propertyChanged;

        public event PropertyChangedEventHandler PropertyChanged
        {
            add { _propertyChanged += value; }
            //// ReSharper disable DelegateSubtraction
            remove { _propertyChanged -= value; }
            //// ReSharper restore DelegateSubtraction
        }

        public int Count => _items.Count;

        public TItem this[string name]
        {
            get
            {
                TItem result;
                _items.TryGetValue(name, out result);
                return result;
            }
        }

        public void Add(string name, TItem item)
        {
            name.CheckNotNull(nameof(name));
            item.CheckNotNull(nameof(item));

            _items.Add(name, item);
            OnPropertyChanged(name);
        }

        public override PropertyDescriptorCollection GetProperties()
        {
            return GetProperties(null);
        }

        public override PropertyDescriptorCollection GetProperties(Attribute[] attributes)
        {
            var properties = _items.Keys.Select(i => new ItemPropertyDescriptor(i, GetType()));
            return new PropertyDescriptorCollection(properties.Cast<PropertyDescriptor>().ToArray(), true);
        }

        public void Dispose()
        {
            var keys = _items.Keys.ToList();
            foreach (var key in keys)
            {
                var item = _items[key] as IDisposable;

                _items.Remove(key);
                OnPropertyChanged(key);

                item?.Dispose();
            }

            _propertyChanged = null;
        }

        private void OnPropertyChanged(string propertyName)
        {
            _propertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
