﻿using System;
using System.Diagnostics.CodeAnalysis;
using System.Runtime.InteropServices;

namespace XCE.SelectionReset
{
    /// <summary>
    ///     Class for generating sequential <see cref="T:System.Guid" />.
    ///     See: http://msdn.microsoft.com/en-us/library/aa379322(VS.85).aspx
    /// </summary>
    public static class SequentialGuid
    {
        /// <summary>
        ///     Generate sequential <see cref="T:System.Guid" />
        /// </summary>
        /// <returns></returns>
        public static Guid NewGuid()
        {
            Guid guid;
            if (SequentialGuid.NativeMethods.UuidCreateSequential(out guid) != 0)
                return Guid.NewGuid();
            byte[] byteArray = guid.ToByteArray();
            byte[] b = new byte[16];
            b[3] = byteArray[0];
            b[2] = byteArray[1];
            b[1] = byteArray[2];
            b[0] = byteArray[3];
            b[5] = byteArray[4];
            b[4] = byteArray[5];
            b[7] = byteArray[6];
            b[6] = byteArray[7];
            b[8] = byteArray[8];
            b[9] = byteArray[9];
            b[10] = byteArray[10];
            b[11] = byteArray[11];
            b[12] = byteArray[12];
            b[13] = byteArray[13];
            b[14] = byteArray[14];
            b[15] = byteArray[15];
            return new Guid(b);
        }

        [SuppressMessage("Microsoft.Performance", "CA1812:AvoidUninstantiatedInternalClasses")]
        private class NativeMethods
        {
            [DllImport("rpcrt4.dll", SetLastError = true)]
            public static extern int UuidCreateSequential(out Guid guid);
        }
    }
}
