﻿using System;
using System.Globalization;
using System.Windows.Data;

namespace XCE.SelectionReset
{
    public abstract class ValueConverterBase<TSource, TTarget, TParameter> : IValueConverter
    {
        public abstract TTarget Convert(TSource value, TParameter parameter, CultureInfo culture);

        public virtual TSource ConvertBack(TTarget value, Type targetType, TParameter parameter, CultureInfo culture)
        {
            throw new NotSupportedException();
        }

        object IValueConverter.Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            var source = Cast(value);
            var p = (TParameter)parameter;

            return Convert(source, p, culture ?? CultureInfo.InvariantCulture);
        }

        object IValueConverter.ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            var target = (TTarget)value;
            var p = (TParameter)parameter;

            return ConvertBack(target, targetType, p, culture ?? CultureInfo.InvariantCulture);
        }

        protected static TSource Cast(object value)
        {
            if (value == null)
            {
                // to avoid System.NullReferenceException when DataContext is deatached (null)
                return default(TSource);
            }

            return (TSource)value;
        }
    }

    public abstract class ValueConverterBase<TSource, TTarget> : ValueConverterBase<TSource, TTarget, object>
    {
        public sealed override TTarget Convert(TSource value, object parameter, CultureInfo culture)
        {
            return Convert(value, culture);
        }

        public sealed override TSource ConvertBack(TTarget value, Type targetType, object parameter, CultureInfo culture)
        {
            return ConvertBack(value, targetType, culture);
        }

        public abstract TTarget Convert(TSource value, CultureInfo culture);

        public virtual TSource ConvertBack(TTarget value, Type targetType, CultureInfo culture)
        {
            throw new NotSupportedException();
        }
    }
}