﻿namespace XCE.SelectionReset
{
    public class FeatureViewModel : ViewModelBase
    {
        private string _name;

        private string _feature;

        public string Name
        {
            get { return _name; }
            set
            {
                if (_name == value) return;
                _name = value;
                OnPropertyChanged();
            }
        }

        public string Feature
        {
            get { return _feature; }
            set
            {
                if (_feature == value) return;
                _feature = value;
                OnPropertyChanged();
                OnPropertyChanged("IsValid");
            }
        }

        public bool? IsValid
        {
            get
            {
                if (Feature.IsNullOrWhiteSpace())
                {
                    return null;
                }

                return IsValidPolygon();
            }  
        } 

        public bool IsValidPolygon()
        {
            // Collapse multiple space characters to one
            var feature = System.Text.RegularExpressions.Regex.Replace(Feature, @"\s+", " ");

            var points = feature.Split(',');
            if (points.Length < 4)
            {
                return false;
            }

            if (points[0].Trim() != points[points.Length - 1].Trim())
            {
                return false;
            }

            foreach (var point in points)
            {
                var coords = point.Trim().Split(' ');
                if (coords.Length != 2)
                {
                    return false;
                }

                float someFloat;
                if (!float.TryParse(coords[0], out someFloat) || !float.TryParse(coords[1], out someFloat))
                {
                    return false;
                }
            }

            return true;
        }
    }
}
