﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;

namespace XCE.SelectionReset
{
    public class LayerAViewModel : LayerBaseViewModel
    {
        private Tonality _tonality;

        private ObservableCollection<FeatureViewModel> _features;

        public override LayerType Type => LayerType.A;

        public IEnumerable<Tonality> Tonalities => Enum.GetValues(typeof(Tonality)).Cast<Tonality>();

        public Tonality Tonality
        {
            get { return _tonality; }
            set
            {
                if (_tonality == value) return;
                _tonality = value;
                OnPropertyChanged();
            }
        }

        public ObservableCollection<FeatureViewModel> Features
        {
            get { return _features; }
            private set
            {
                if (_features == value) return;
                _features = value;
                OnPropertyChanged();
            }
        }

        public LayerAViewModel(LayerA layerAModel)
        {
            Commands.Add("AddFeature", AddFeature);
            Commands.Add<FeatureViewModel>("DeleteFeature", DeleteFeature);
            Features = new ObservableCollection<FeatureViewModel>();
            Initialize(layerAModel);
        }

        private void Initialize(LayerA layerAModel)
        {
            Id = layerAModel.Id;
            Name = layerAModel.Name;
            Tonality = layerAModel.Tonality;
            Order = layerAModel.Order;

            foreach (var featureModel in layerAModel.Features)
            {
                Features.Add(new FeatureViewModel
                {
                    Name = featureModel.Name,
                    Feature = ExtractFeature(featureModel.Polygon)
                });
            }
        }

        public LayerA CreateModel()
        {
            var model = new LayerA
            {
                Id = Id,
                Name = Name,
                Tonality = Tonality,
                Order = Order
            };

            foreach (var feature in Features)
            {
                model.Features.Add(new Feature
                {
                    Id = SequentialGuid.NewGuid(),
                    Name = feature.Name,
                    Polygon = $"POLYGON(({feature.Feature}))"
                });
            }

            return model;
        }

        private string ExtractFeature(string wktPolygon)
        {
            // POLYGON((...))  ->  ...
            var from = wktPolygon.LastIndexOf("(") + 1;
            var to = wktPolygon.IndexOf(")");
            return wktPolygon.Substring(from, to - from);
        }

        private void AddFeature()
        {
            Features.Add(new FeatureViewModel { Name = GenerateFeatureName() });
        }

        private void DeleteFeature(FeatureViewModel feature)
        {
            Features.Remove(feature);
        }

        private string GenerateFeatureName()
        {
            var no = Features.Count;
            while (true)
            {
                var name = $"Feature {++no}";
                if (Features.All(feature => !feature.Name.EqualsIgnoreCase(name)))
                {
                    return name;
                }
            }
        }
    }
}
