﻿using System.Collections.ObjectModel;
using System.Linq;

namespace XCE.SelectionReset
{
    public class LayerBViewModel : LayerBaseViewModel
    {
        private ObservableCollection<object> _targetLayers;

        public override LayerType Type => LayerType.B;

        public ObservableCollection<LayerBaseViewModel> AllLayers { get; set; }

        public ObservableCollection<TargetLayerViewModel> LowerLayers
            => new ObservableCollection<TargetLayerViewModel>(from l in AllLayers
                where l.Order < Order
                select new TargetLayerViewModel
                {
                    Layer = l
                });

        public ObservableCollection<object> TargetLayers
        {
            get { return _targetLayers; }
            set
            {
                if (_targetLayers == value) return;
                _targetLayers = value;
                OnPropertyChanged();
            }
        }

        public LayerBViewModel(LayerB layerBModel,
            ObservableCollection<LayerBaseViewModel> allLayers)
        {
            Initialize(layerBModel, allLayers);
        }

        private void Initialize(LayerB layerBModel,
            ObservableCollection<LayerBaseViewModel> allLayers)
        {
            Id = layerBModel.Id;
            Name = layerBModel.Name;
            Order = layerBModel.Order;

            AllLayers = allLayers;
            TargetLayers = new ObservableCollection<object>();
            foreach (var id in layerBModel.TargetLayerIds)
            {
                TargetLayers.Add(new TargetLayerViewModel {Layer = allLayers.First(l => l.Id == id)});
            }
        }

        public LayerB CreateModel()
        {
            var model = new LayerB
            {
                Id = Id,
                Name = Name,
                Order = Order
            };

            foreach (TargetLayerViewModel layer in TargetLayers)
            {
                model.TargetLayerIds.Add(layer.Id);
            }

            return model;
        }
    }
}