using System.Collections.ObjectModel;
using System.Linq;

namespace XCE.SelectionReset
{
    public class LayerCViewModel : LayerBaseViewModel
    {
        private ObservableCollection<object> _layersSet1;

        private ObservableCollection<object> _layersSet2;

        public override LayerType Type => LayerType.C;

        public ObservableCollection<LayerBaseViewModel> AllLayers { get; set; }

        public ObservableCollection<TargetLayerViewModel> LowerLayers => new ObservableCollection<TargetLayerViewModel>(from l in AllLayers
                                                                                                                        where l.Order < Order
                                                                                                                        select new TargetLayerViewModel
                                                                                                                        {
                                                                                                                            Layer = l
                                                                                                                        });

        public ObservableCollection<object> LayersSet1
        {
            get { return _layersSet1; }
            set
            {
                if (_layersSet1 == value) return;
                _layersSet1 = value;
                OnPropertyChanged();
            }
        }

        public ObservableCollection<object> LayersSet2
        {
            get { return _layersSet2; }
            set
            {
                if (_layersSet2 == value) return;
                _layersSet2 = value;
                OnPropertyChanged();
            }
        }

        public LayerCViewModel(LayerC layerCModel, ObservableCollection<LayerBaseViewModel> allLayers)
        {

            Initialize(layerCModel, allLayers);
        }

        private void Initialize(LayerC layerCModel, ObservableCollection<LayerBaseViewModel> allLayers)
        {
            Id = layerCModel.Id;
            Name = layerCModel.Name;
            Order = layerCModel.Order;

            AllLayers = allLayers;
            LayersSet1 = new ObservableCollection<object>();
            LayersSet2 = new ObservableCollection<object>();
            foreach (var id in layerCModel.LayerIds1)
            {
                LayersSet1.Add(new TargetLayerViewModel { Layer = allLayers.First(target => target.Id == id) });
            }

            foreach (var id in layerCModel.LayerIds2)
            {
                LayersSet2.Add(new TargetLayerViewModel { Layer = allLayers.First(target => target.Id == id) });
            }
        }

        public LayerC CreateModel()
        {
            var model = new LayerC
            {
                Id = Id,
                Name = Name,
                Order = Order
            };

            foreach (TargetLayerViewModel layer in LayersSet1)
            {
                    model.LayerIds1.Add(layer.Id);
            }

            foreach (TargetLayerViewModel layer in LayersSet2)
            {
                    model.LayerIds2.Add(layer.Id);
            }

            return model;
        }
    }
}
